"use strict";exports.__esModule = true;exports.isNativeTag = isNativeTag;exports.isDynamicTag = isDynamicTag;exports.isAttributeTag = isAttributeTag;exports.isTransparentTag = isTransparentTag;exports.isMacroTag = isMacroTag;exports.getMacroIdentifier = getMacroIdentifier;exports.getTagDef = getTagDef;exports.getFullyResolvedTagName = getFullyResolvedTagName;exports.findParentTag = findParentTag;exports.findAttributeTags = findAttributeTags;exports.getArgOrSequence = getArgOrSequence;var _babelTypes = require("@marko/babel-types");
const transparentTags = new Set(["for", "while", "if", "else", "_no-update"]);

function isNativeTag(path) {
  const tagDef = path.node.tagDef;
  return (
    tagDef &&
    tagDef.html && (
    tagDef.htmlType === "custom-element" ||
    !tagDef.template && !tagDef.renderer));

}

function isDynamicTag(path) {
  return !path.get("name").isStringLiteral();
}

function isAttributeTag(path) {
  return !isDynamicTag(path) && path.get("name.value").node[0] === "@";
}

function isTransparentTag(path) {
  return (
    !isDynamicTag(path) && transparentTags.has(path.get("name.value").node));

}

function isMacroTag(path) {
  return Boolean(getMacroIdentifier(path));
}

function getMacroIdentifier(path) {
  return (
    !isDynamicTag(path) && path.hub.file._macros[path.get("name.value").node]);

}

function getTagDef(path) {
  const cached = path.get("tagDef");

  if (cached.node !== undefined) {
    return cached.node;
  }

  const {
    hub: { file } } =
  path;
  const { _lookup } = file;
  let tagName;

  if (!(isMacroTag(path) || isDynamicTag(path))) {
    tagName = isAttributeTag(path) ?
    getFullyResolvedTagName(path) :
    path.get("name.value").node;
  }

  const tagDef = tagName && _lookup.getTag(tagName) || null;
  path.set("tagDef", tagDef);
  return tagDef;
}

function getFullyResolvedTagName(path) {
  const parts = [];
  let cur;
  do {
    cur = path.node.name.value;

    if (isAttributeTag(path)) {
      parts.push(cur.slice(1));
    } else {
      parts.push(cur || "*");
      break;
    }
  } while (path = findParentTag(path));

  return parts.reverse().join(":");
}

function findParentTag(path) {
  let cur = path.parentPath;

  while (cur.node) {
    if (cur.isMarkoTagBody()) {
      cur = cur.parentPath;
      continue;
    }

    if (!cur.isMarkoTag()) {
      cur = undefined;
      break;
    }

    if (isTransparentTag(cur)) {
      cur = cur.parentPath;
      continue;
    }

    return cur;
  }
}

function findAttributeTags(path, attributeTags = []) {
  path.get("body.body").forEach(child => {
    if (isAttributeTag(child)) {
      attributeTags.push(child);
    } else if (isTransparentTag(child)) {
      findAttributeTags(child, attributeTags);
    }
  });

  return attributeTags;
}

function getArgOrSequence(path) {
  const {
    node: { arguments: args } } =
  path;
  const len = args && args.length;

  if (len) {
    if (len > 1) {
      return _babelTypes.types.sequenceExpression(args);
    } else {
      return args[0];
    }
  }
}
//# sourceMappingURL=tags.js.map